CodeBook

Struct CodeBook 

pub struct CodeBook<T: Hash + Eq> { /* private fields */ }
A mapping of huffman codes to data symbols.

Implementations

Source

impl<T: Hash + Eq + Clone + Ord + Debug> CodeBook<T>

pub fn from_data(data: impl AsRef<[T]>) -> Result<Self, Error>

Constructs a code book optimal over the given symbols.

The codebook will provide the optimal single-symbol prefix-free mapping, and can be used to encode or decode data according to that mapping.

Arguments
  • data - A slice of symbols.
Errors
  • Error::DataEmpty if data contains 0 or 1 distinct symbols, since there is no sense in encoding this
pub fn from_mapping(mapping: impl AsRef<[(T, HfmnCode)]>) -> Self

Returns a codebook crated from a supplied mapping.

Arguments
  • mapping - A slice of (symbol, code). This must contain each symbol at most once, and each ocde must be valid, and the set of codes must be correctly formed.
pub fn encode_data( &self, data: impl AsRef<[T]>, ) -> Result<Vec<&HfmnCode>, Error>

Encodes symbols using the calculated Huffman codebook

Returns both the encoded version of the symbols and the total number of symbols encoded. This is needed by the decoder in order to accurately decode the codestream.

Arguments
  • data - The symbols to be encoded
Errors
  • Error::SymbolNotFound if any of the symbols in data are not in the codebook
pub fn decode_data( &self, bytes: &[u8], num_symbols: usize, ) -> Result<Vec<&T>, Error>

Decode huffman codes

Arguments
  • bytes - The bytes containing the codestream
  • num_symbols - The number of symbols that were encoded
Errors
  • Error::InvalidCode if any of the codes read are not found in the codebook
Source

impl<'a, T: Serialize + Deserialize<'a> + Hash + Ord + Clone + Debug> CodeBook<T>

pub fn encode_book(self) -> Vec<u8>

Serialize the codebook to bytes

pub fn decode_book(bytes: &'a [u8]) -> Result<Self, Error>

Deserialize a codebook from bytes

Note this function is not intended to provide deep validation on whether the codebook is valid or not.

Arguments
  • bytes - The bytes to deserialize back into a codebook
Errors
  • Error::BinaryCoding - If the bytes can’t be decoded into a trivially valid codebook