pub struct CodeBook<T: Hash + Eq> { /* private fields */ }Expand description
A mapping of huffman codes to data symbols.
Implementations
Sourceimpl<T: Hash + Eq + Clone + Ord + Debug> CodeBook<T>
impl<T: Hash + Eq + Clone + Ord + Debug> CodeBook<T>
Sourcepub fn from_data(data: impl AsRef<[T]>) -> Result<Self, Error>
pub fn from_data(data: impl AsRef<[T]>) -> Result<Self, Error>
Constructs a code book optimal over the given symbols.
The codebook will provide the optimal single-symbol prefix-free mapping, and can be used to encode or decode data according to that mapping.
Arguments
data- A slice of symbols.
Errors
Error::DataEmptyifdatacontains 0 or 1 distinct symbols, since there is no sense in encoding this
Sourcepub fn from_mapping(mapping: impl AsRef<[(T, HfmnCode)]>) -> Self
pub fn from_mapping(mapping: impl AsRef<[(T, HfmnCode)]>) -> Self
Returns a codebook crated from a supplied mapping.
Arguments
mapping- A slice of (symbol, code). This must contain each symbol at most once, and each ocde must be valid, and the set of codes must be correctly formed.
Sourcepub fn encode_data(
&self,
data: impl AsRef<[T]>,
) -> Result<Vec<&HfmnCode>, Error>
pub fn encode_data( &self, data: impl AsRef<[T]>, ) -> Result<Vec<&HfmnCode>, Error>
Encodes symbols using the calculated Huffman codebook
Returns both the encoded version of the symbols and the total number of symbols encoded. This is needed by the decoder in order to accurately decode the codestream.
Arguments
data- The symbols to be encoded
Errors
Error::SymbolNotFoundif any of the symbols in data are not in the codebook